package top.ninwoo.edgecenter.service.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import top.ninwoo.common.entity.NetworkTopology;
import top.ninwoo.edgecenter.service.ClusterService;
import top.ninwoo.edgecenter.service.IpService;
import top.ninwoo.edgecenter.service.LogicTopoService;
import top.ninwoo.utils.service.IptablesService;

import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

@Service
public class LogicTopoServiceImpl implements LogicTopoService {
    private final static Logger LOG = LoggerFactory.getLogger(TopologyServiceImpl.class);
    // 用于存储集群的拓扑
    private ConcurrentHashMap<Long, NetworkTopology> clustersTopo = new ConcurrentHashMap<>();

    @Value("${bs.sdn-controller.host}")
    private String host;
    @Value("${bs.sdn-controller.port}")
    private int port;

    @Autowired
    ClusterService clusterService;

    @Autowired
    IpService ipService;

    @Autowired
    IptablesService iptablesService;

    @Override
    public void creatLogicTopo(long clusterId, NetworkTopology oldtopo) {
        NetworkTopology logictopo = this.getLogictopo(oldtopo);

        if (logictopo == null) {
            LOG.warn("集群[" + clusterId + "]未设置网络");
            return;
        }

        // 校验信息
        String[] cNames = logictopo.getAppNames();
        int[][] topo = logictopo.getTopology();
        if (cNames == null || cNames.length <= 1) {
            return;
        }

        // 这里声明：cNames中不包含ovs虚拟机，仅包含docker容器组成的逻辑拓扑
        for (int i = 1; i < cNames.length; i++) {
            for (int j = 0; j < i; j++) {
                // 判断每一个节点的连接状态
                if (topo[i][j] == 0) {
                    // 如果判断为0，添加IPtabale，使其断开
                    this.addDropIPtable(clusterId, cNames[i], cNames[j]);
                }
            }
        }
        clustersTopo.put(clusterId, logictopo);
    }

    @Override
    public String modifyLogicTopology(long clusterId, NetworkTopology oldtopo) {
        //System.out.println("我运行了么？");
        NetworkTopology logictopo = this.getLogictopo(oldtopo);
        // 先进行校验
        if (logictopo == null) {
            System.out.println("topoloy不能为空");
            return "topoloy不能为空";
        }

        // 获取clusterId的网络拓扑
        if (!clustersTopo.containsKey(clusterId)) {
            System.out.println("该集群没有设置topo");
            return "该集群没有设置topo";
        }

        NetworkTopology origTopo = clustersTopo.get(clusterId);
        // 校验topoId是否一致
        if (origTopo.getTopologyId() != logictopo.getTopologyId()) {
            System.out.println("不是相同的网络拓扑，无法更改");
            return "不是相同的网络拓扑，无法更改";
        }

        // 校验成员名称是否相同
        for (int i = 0; i < origTopo.getAppNames().length; i++) {
            if (!origTopo.getAppNames()[i].equals(logictopo.getAppNames()[i])) {
                System.out.println("App名称必须一致");
                return "App名称必须一致";
            }
        }

        int[][] origTopoArr = origTopo.getTopology();
        int[][] newTopoArr = logictopo.getTopology();
        //System.out.println("康康我的新拓扑怎样呢");
        //System.out.println(newTopoArr[2][1]);
        // 校验topo是否一致
        if ((origTopoArr.length != newTopoArr.length)
                || (origTopoArr[0].length != newTopoArr[0].length)) {
            System.out.println("拓扑大小不一致");
            return "拓扑大小不一致";
        }

        //System.out.println("终于要更新网络了啦");
        // 更新网络拓扑
        for (int i = 1; i < newTopoArr.length; i++) {
            for (int j = 0; j < i; j++) {
                //增加新的 dropIPtable
                if (origTopoArr[i][j] == 1 && newTopoArr[i][j] == 0) {
                    // 创建新的连接
                    this.addDropIPtable(clusterId, origTopo.getAppNames()[i], origTopo.getAppNames()[j]);
                    //System.out.println("改了一次");
                } else if (origTopoArr[i][j] == 0 && newTopoArr[i][j] == 1) {
                    // 删除旧的连接
                    this.addDelDropIPtable(clusterId, origTopo.getAppNames()[i], origTopo.getAppNames()[j]);
                    //System.out.println("gaile");
                }
            }
        }
        clustersTopo.put(clusterId, logictopo);

        return "success";
    }

    @Override
    public NetworkTopology getLogictopo(NetworkTopology oritopo) {
        NetworkTopology logictopo = new NetworkTopology();

        int flag = 0;
        //获取appnames
        for (int i = 0; i < oritopo.getAppNames().length - 1; i++) {
            if (oritopo.getAppNames()[i].startsWith("br:")) {
                break;
            }
            flag++;
        }
        String[] appname = new String[flag];
        for(int k=0;k<flag;k++){appname[k]=oritopo.getAppNames()[k];}
        logictopo.setAppNames(appname);

        //获取逻辑拓扑
        int[][] logicTopo = new int[flag][flag];
        for (int j=1;j<flag;j++){
            for (int m=0;m<j;m++){logicTopo[j][m] = oritopo.getTopology()[j][m];}
        }
        logictopo.setTopology(logicTopo);

        logictopo.setTopologyId(oritopo.getTopologyId());

        return logictopo;
    }


    public void addDelDropIPtable(long clusterId, String appName1, String appName2) {
        //获取容器ID
        Set<String> cid1 = clusterService.getContainerIdsByClusterId(clusterId, appName1);
        String appID1 = ((String) cid1.toArray()[0]);
        String appIP1 = ipService.getContainerIp(appID1).split("/")[0];
        Set<String> cid2 = clusterService.getContainerIdsByClusterId(clusterId, appName2);
        String appID2 = ((String) cid2.toArray()[0]);
        String appIP2 = ipService.getContainerIp(appID2).split("/")[0];
        //dropTraffic(String containerId, String sourceIp, String destinationIp)
        //dropTraffic模块      容器ID       源（容器）地址      目的（容器）地址
        iptablesService.cancelDropTraffic(appID1, appIP2, appIP1);
        iptablesService.cancelDropTraffic(appID2, appIP1, appIP2);
    }

    public void addDropIPtable(long clusterId, String appName1, String appName2) {
        //获取容器ID
        Set<String> cid1 = clusterService.getContainerIdsByClusterId(clusterId, appName1);
        String appID1 = ((String) cid1.toArray()[0]);
        String appIP1 = ipService.getContainerIp(appID1).split("/")[0];
        Set<String> cid2 = clusterService.getContainerIdsByClusterId(clusterId, appName2);
        String appID2 = ((String) cid2.toArray()[0]);
        String appIP2 = ipService.getContainerIp(appID2).split("/")[0];
        //dropTraffic(String containerId, String sourceIp, String destinationIp)
        //dropTraffic模块      容器ID       源（容器）地址      目的（容器）地址
        iptablesService.dropTraffic(appID1, appIP2, appIP1);
        iptablesService.dropTraffic(appID2, appIP1, appIP2);
    }
}
